use std::{panic, process};
use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};

pub fn exit_on_panic() {
    let orig_hook = panic::take_hook();
    panic::set_hook(Box::new(move |panic_info| {
        // invoke the default handler and exit the process
        orig_hook(panic_info);
        process::exit(1);
    }));
}

#[cfg(debug)]
const DEFAULT_LOG_LEVEL: &str = "debug";
#[cfg(not(debug))]
const DEFAULT_LOG_LEVEL: &str = "info";

pub fn init_logger(_server_port: u16) {
    tracing_subscriber::registry()
        .with(
            tracing_subscriber::EnvFilter::try_from_default_env().unwrap_or_else(|_| {
                DEFAULT_LOG_LEVEL.into()
            }),
        )
        .with(tracing_subscriber::fmt::layer())
        .init();
}
